/*
* ============================================================================
*  Name        : mccexappaudioctrlview.h
*  Part of     : mccexample / mccexapp
*  Description : CMccExAppAudioCtrlView declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CMCCEXAPPAUDIOCTRLVIEW_H
#define C_CMCCEXAPPAUDIOCTRLVIEW_H

#include "mccexappviewbase.h"
#include "mmccexengineobserver.h"

class CMccExEngine;
class MAudioControl;

/**
 * CMccExAppAudioCtrlView is CMccExAppViewBase/CAknView based object providing
 * a separate view for audio streaming functionality.
 */
class CMccExAppAudioCtrlView : public CMccExAppViewBase,
    public MMccExEngineObserver
    {
public:
// Constructors and destructor

    /**
     * Destructor.
     */
    virtual ~CMccExAppAudioCtrlView();

    /**
     * Two-phased constructor.
     * 
     * @param aEngine Reference to the engine.
     */
    static CMccExAppAudioCtrlView* NewL( CMccExEngine& aEngine );

    /**
     * Two-phased constructor. Leaves the created instance into the cleanup
     * stack.
     * 
     * @param aEngine Reference to the engine.
     */
    static CMccExAppAudioCtrlView* NewLC( CMccExEngine& aEngine );

// From MMccExEngineObserver
    
    /**
     * From MMccExEngineObserver.
     * Network connection setup start notification.
     * 
     * @see MMccExEngineObserver
     */
    void NetworkSetupStarted();
    
    /**
     * From MMccExEngineObserver.
     * Network connection setup ready notification.
     * 
     * @see MMccExEngineObserver
     */
    void NetworkSetupReady( TInt aError );
    
    /**
     * From MMccExEngineObserver.
     * State change complete notification.
     * 
     * @see MMccExEngineObserver
     */
    void StateChangeComplete( TControlState aState, TInt aError );
    
    /**
     * From MMccExEngineObserver.
     * Error detection notification.
     * 
     * @see MMccExEngineObserver
     */
    void ErrorDetected( TControlState aState, TInt aError );
    
// From CAknView
    
    /**
     * From CAknView.
     * Handles the menu commands for this view.
     * 
     * @see CAknView
     */
    void HandleCommandL( TInt aCommand );
    
// From MEikMenuObserver
    
    /**
     * From MEikMenuObserver.
     * Handles the dynamic initialization of the menu pane.
     * 
     * @see MEikMenuObserver
     */
    void DynInitMenuPaneL( TInt aResourceId, CEikMenuPane* aMenuPane );
    
protected:
    
// From CMccExAppViewBase
    
    /**
     * From CMccExAppViewBase.
     * Factory method for creating the compound control of this view. Every
     * deriving class must implement this function.
     * 
     * @see CMccExAppViewBase
     * @return CCoeControl Newly created compound control.
    */
    CCoeControl* CreateContainerL();
    
// From CAknView
    
    /**
     * From CAknView.
     * Handles the view activation.
     * 
     * @see CAknView
     */
    void DoActivateL( const TVwsViewId& aPrevViewId,
            TUid aCustomMessageId, const TDesC8& aCustomMessage );
    
    /**
     * From CAknView.
     * Handles the view deactivation.
     * 
     * @see CAknView
     */
    void DoDeactivate();
    
private:
    
    /**
     * Constructor for performing 1st stage construction.
     * 
     * @param aEngine Reference to the engine.
     */
    CMccExAppAudioCtrlView( CMccExEngine& aEngine );
    
    /**
     * Command handling for preparing an audio control.
     */
    void PrepareAudioControlCmdL();
    
    /**
     * Command handling for starting an audio control.
     */
    void StartAudioControlCmdL();
    
    /**
     * Command handling for pausing an audio control.
     */
    void PauseAudioControlCmdL();
    
    /**
     * Command handling for resuming an audio control.
     */
    void ResumeAudioControlCmdL();
    
    /**
     * Command handling for stopping an audio control.
     */
    void StopAudioControlCmdL();
    
    /**
     * Command handling for changing audio routing.
     */
    void ChangeAudioRouteCmdL();
    
    /**
     * Command handling for mic mute.
     */
    void ChangeMuteStateCmdL();
    
    /**
     * Worker function for dynamically handling menu pane initialization.
     * 
     * @param aMenu The menu pane.
     */
    void DoMenuPaneInitL( CEikMenuPane* aMenu );
    
    /**
     * Worker function for handling audio route and mute change menu option
     * dynamically.
     * 
     * @param aMenu The menu pane.
     */
    void HandleRouteAndMuteOptionL( CEikMenuPane* aMenu ) const;
    
    /**
     * Helper function for determining resource ID for the current state.
     * 
     * @param aState Current state.
     * @return TInt Resource ID for the associated string.
     */
    TInt DetermineStateStringResource( TControlState aState );
    
private:    // Data
    
    /**
     * Audio control.
     * Own
     */
    MAudioControl* iLoopbackControl;
    
    /**
     * Stores audio routing data. Default routing is to IHF.
     */
    TUint32 iAudioRouting;
    
    /**
     * Flag to indicate mute status. ETrue if mute is ON/enabled.
     */
    TBool iMuteState;
    
    };

#endif // C_CMCCEXAPPAUDIOCTRLVIEW_H
