/*
* ============================================================================
*  Name        : mccexappcodecsettinglist.h
*  Part of     : mccexample / mccexapp
*  Description : CMccExAppCodecSettingList declaration
*  Version     : %version: %
*
*  Copyright  2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CMCCEXAPPCODECSETTINGLIST_H
#define C_CMCCEXAPPCODECSETTINGLIST_H

#include <aknsettingitemlist.h>

class CMccExAppCodecSettings;


/**
 * CMccExAppCodecSettingList implements the codec setting item list control.
 * It encapsulates all the needed functionality to edit, update and save
 * the setting data related to MCC example application codec settings.
 */
class CMccExAppCodecSettingList : public CAknSettingItemList
    {
public:
    
// Constructors and destructor
    
    /**
     * Two phased constructor.
     * 
     * @return Pointer to a newly created object.
     */
    static CMccExAppCodecSettingList* NewL(
        CMccExAppCodecSettings& aSettings );
    
    /**
     * Two-phased constructor. Leaves the created object into cleanup stack.
     * 
     * @return Pointer to a newly created object.
     */
    static CMccExAppCodecSettingList* NewLC(
        CMccExAppCodecSettings& aSettings );
    
    /**
     * Destructor.
     */
    virtual ~CMccExAppCodecSettingList();
    
// From CAknSettingItemList
    
    /**
     * From CAknSettingItemList.
     * Handles the item editing. Overridden here for custom functionality.
     * 
     * @see CAknSettingItemList
     * @param aIndex Index of the setting item.
     * @param aCalledFromMenu Flag to indicate whether or not being called
     *                        from menu.
     */
    void EditItemL( TInt aIndex, TBool aCalledFromMenu );
    
public:
    
// New functions.
    
    /**
     * Helper function for changing selected item of the setting list. Called
     * by CMccExAppCodecSettingView when user selects "Change" from the menu.
     */ 
    void ChangeSelectedItemL();
    
    /**
     * Helper function for populating the codec list dynamically.
     * Called by CMccExAppCodecSettingView when creating the setting list.
     */
    void PopulateCodecListL();

protected:
    
// From CAknSettingItemList
    
    /**
     * From CAknSettingItemList.
     * Factory function for creating a setting item matching the given ID.
     * 
     * @see CAknSettingItemList
     * @param aSettingId ID of the setting to be created.
     */
    CAknSettingItem* CreateSettingItemL( TInt aSettingId );
    
// From CCoeControl
    
    /**
     * From CCoeControl.
     * Size changed.
     * 
     * @see CCoeControl.
     */
    void SizeChanged();
        
private:
    
    /**
     * C++ constructor.
     */
    CMccExAppCodecSettingList( CMccExAppCodecSettings& aSettings );
    
    /**
     * Helper function for loading the actual codec setting list items to the
     * associated control.
     * 
     * @param aItem The setting item for codec list.
     */
    void DoPopulateCodecListL( CAknEnumeratedTextSettingItem* aItem );
    
    /**
     * Helper function for handling selected codec index update.
     * 
     * @param aItem The setting item for codec list.
     */
    void HandleCodecSelectionChangeL( CAknEnumeratedTextSettingItem* aItem );

private:
    
    /**
     * Reference to the codec settings model.
     */
    CMccExAppCodecSettings& iSettings;
    
    };

#endif // C_CMCCEXAPPCODECSETTINGLIST_H
