/*
* ============================================================================
*  Name        : mccexappvideoctrlview.h
*  Part of     : mccexample / mccexapp
*  Description : CMccExAppVideoCtrlView declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CMCCEXAPPVIDEOCTRLVIEW_H
#define C_CMCCEXAPPVIDEOCTRLVIEW_H

#include "mccexappviewbase.h"
#include "mmccexengineobserver.h"

class CMccExEngine;
class MVideoControl;

/**
 * CMccExAppVideoCtrlView is CMccExAppViewBase/CAknView based object providing
 * a separate view for video streaming functionality.
 */
class CMccExAppVideoCtrlView : public CMccExAppViewBase,
    public MMccExEngineObserver
    {
public:
    // Constructors and destructor

    /**
     * Destructor.
     */
    virtual ~CMccExAppVideoCtrlView();

    /**
     * Two-phased constructor.
     * 
     * @param aEngine Reference to the engine.
     */
    static CMccExAppVideoCtrlView* NewL( CMccExEngine& aEngine );

    /**
     * Two-phased constructor. Leaves the created instance into the cleanup
     * stack.
     * 
     * @param aEngine Reference to the engine.
     */
    static CMccExAppVideoCtrlView* NewLC( CMccExEngine& aEngine );

// From MMccExEngineObserver
    
    /**
     * From MMccExEngineObserver.
     * Network connection setup start notification.
     * 
     * @see MMccExEngineObserver
     */
    void NetworkSetupStarted();
    
    /**
     * From MMccExEngineObserver.
     * Network connection setup ready notification.
     * 
     * @see MMccExEngineObserver
     */
    void NetworkSetupReady( TInt aError );
    
    /**
     * From MMccExEngineObserver.
     * State change complete notification.
     * 
     * @see MMccExEngineObserver
     */
    void StateChangeComplete( TControlState aState, TInt aError );
    
    /**
     * From MMccExEngineObserver.
     * Error detection notification.
     * 
     * @see MMccExEngineObserver
     */
    void ErrorDetected( TControlState aState, TInt aError );
    
// From CAknView
    
    /**
     * From CAknView.
     * Handles the menu commands for this view.
     * 
     * @see CAknView
     */
    void HandleCommandL( TInt aCommand );
    
// From MEikMenuObserver
    
    /**
     * From MEikMenuObserver.
     * Handles the dynamic initialization of the menu pane.
     * 
     * @see MEikMenuObserver
     */
    void DynInitMenuPaneL( TInt aResourceId, CEikMenuPane* aMenuPane );
    
protected:
    
// From CMccExAppViewBase
    
    /**
     * From CMccExAppViewBase.
     * Factory method for creating the compound control of this view. Every
     * deriving class must implement this function.
     * 
     * @see CMccExAppViewBase
     * @return CCoeControl Newly created compound control.
    */
    CCoeControl* CreateContainerL();
    
// From CAknView
    
    /**
     * From CAknView.
     * Handles the view activation.
     * 
     * @see CAknView
     */
    void DoActivateL( const TVwsViewId& aPrevViewId,
            TUid aCustomMessageId, const TDesC8& aCustomMessage );
    
    /**
     * From CAknView.
     * Handles the view deactivation.
     * 
     * @see CAknView
     */
    void DoDeactivate();
    
private:
    
    /**
     * Constructor for performing 1st stage construction.
     * 
     * @param aEngine Reference to the engine.
     */
    CMccExAppVideoCtrlView( CMccExEngine& aEngine );
    
    /**
     * Helper function for handling 'prepare' menu command.
     */
    void HandleCmdPrepareL();
    
    /**
     * Helper function for handling 'start' menu command.
     */
    void HandleCmdStartL();
    
    /**
     * Helper function for handling 'stop' menu command.
     */
    void HandleCmdStopL();
    
    /**
     * Helper function for handling 'pause' menu command.
     */
    void HandleCmdPauseL();
    
    /**
     * Helper function for handling 'resume' menu command.
     */
    void HandleCmdResumeL();
    
    /**
     * Helper function for dynamically handling menu pane initialization.
     * 
     * @param aMenu The menu pane
     */
    void DoMenuPaneInitL( CEikMenuPane* aMenu );
    
    /**
     * Helper function for handling network setup start event.
     */
    void DoNetworkSetupStartL();
    
    /**
     * Helper function for handling network setup ready event.
     * 
     * @param aError Error code associated to the event.
     */
    void DoNetworkSetupReadyL( TInt aError );
    
    /**
     * Helper function for handling state change complete event.
     * 
     * @param aState Current state.
     * @param aError Error code associated to the event.
     */
    void DoStateChangeCompleteL( TControlState aState, TInt aError );
    
    /**
     * Helper function for handling error detection event.
     * 
     * @param aState Current state.
     * @param aError Error code associated to the event.
     */
    void DoErrorDetectedL( TControlState aState, TInt aError );
    
private:    // Data
    
    /**
     * Video control interface.
     * Own
     */
    MVideoControl* iVideoControl;
    
    };

#endif // C_CMCCEXAPPVIDEOCTRLVIEW_H
