/*
* ============================================================================
*  Name        : mccexappviewbase.h
*  Part of     : mccexample / mccexapp
*  Description : CMccExAppViewBase declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CMCCEXAPPVIEWBASE_H
#define C_CMCCEXAPPVIEWBASE_H

#include <aknview.h>

class CMccExEngine;
class CAknInfoPopupNoteController;

/**
 * CMccExAppViewBase provides a application specific base class for views.
 * Class derives from CAknView and implements some basic functionality common
 * to all application views.
 */
class CMccExAppViewBase : public CAknView
    {
public:

    /**
     * Destructor.
     */
    virtual ~CMccExAppViewBase();
    
    /**
     * Accessor function to the engine.
     * 
     * @return CMccExEngine Returns a reference to the engine.
     */
    CMccExEngine& Engine();
    
    /**
     * Function for AppUi notifying about resource changes. Currently used 
     * especially for dynamic layout changes. If other specific handling is
     * required, the derived class should implement it.
     * 
     * @param aType Type of resource change
     */
    void HandleResourceChange( TInt aType );
    
// From CAknView
    
    /**
     * From CAknView.
     * Returns the ID of this view.
     * 
     * @see CAknView
     */
    TUid Id() const;
    
protected:
    
    /**
     * Constructor for performing 1st stage construction.
     * 
     * @param aEngine Reference to the engine.
     * @param aUid UID of this view.
     */
    CMccExAppViewBase( CMccExEngine& aEngine, const TUid& aUid );
    
    /**
     * Pure virtual factory method for creating the compound control of this
     * view. Every deriving class must implement this function.
     * 
     * @return CCoeControl Newly created compound control.
     */
    virtual CCoeControl* CreateContainerL() = 0;
    
    /**
     * Returns a reference to the container of this view. Must be created
     * using CreateContainerL or this function panics.
     * 
     * @return CCoeControl Compound control associated for this view
     */
    CCoeControl& Control();
    
    /**
     * Helper function for showing a information note.
     * 
     * @param aMessage Message to be shown in the information note.
     */
    void DoShowInfoNoteL( const TDesC& aMessage );
    
    /**
     * Helper function for showing a information pop-up.
     * 
     * @param aMessage Message to be shown in the information pop-up.
     */
    void DoShowInfoPopupL( const TDesC& aMessage );
    
// From CAknView
    
    /**
     * From CAknView.
     * Handles the view activation.
     * 
     * @see CAknView
     */
    virtual void DoActivateL( const TVwsViewId& aPrevViewId,
            TUid aCustomMessageId, const TDesC8& aCustomMessage );
    
    /**
     * From CAknView.
     * Handles the view deactivation.
     * 
     * @see CAknView
     */
    virtual void DoDeactivate();

private:
    
    /**
     * Helper function for setting the status pane caption for the view.
     * 
     * @param aResource Resource ID of the caption string.
     */
    void SetCaptionTextL( TInt aResource );
    
private:  // Data
    
    /**
     * Reference to the engine.
     */
    CMccExEngine& iEngine;
    
    /**
     * Compound control of this view.
     * Own.
     */
    CCoeControl* iContainer;
    
    /**
     * Information note controller.
     * Own.
     */
    CAknInfoPopupNoteController* iNoteController;
    
    /**
     * UID of this view.
     */
    TUid iUid;
    
    };

#endif // C_CMCCEXAPPVIEWBASE_H
