/*
* ============================================================================
*  Name        : mccexappbasecontrol.cpp
*  Part of     : mccexample / mccexapp
*  Description : MCC example application CMccExAppBaseControl implementation
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.1.1
*/

#include <aknutils.h>
#include <aknsdrawutils.h>
#include <aknsbasicbackgroundcontrolcontext.h>

#include "mccexappbasecontrol.h"

// ======== MEMBER FUNCTIONS ========


// ---------------------------------------------------------------------------
// CMccExAppBaseControl::BaseConstructL
// ---------------------------------------------------------------------------
//
void CMccExAppBaseControl::BaseConstructL( const TRect& aRect )
    {
    CreateWindowAndBackgroundL( aRect );
    SetRectAndActivateL( aRect );
    }


// ---------------------------------------------------------------------------
// CMccExAppBaseControl::CreateWindowAndBackgroundL
// ---------------------------------------------------------------------------
//
void CMccExAppBaseControl::CreateWindowAndBackgroundL( const TRect& aRect )
    {
    iBgContext = CAknsBasicBackgroundControlContext::NewL(
            KAknsIIDQsnBgAreaMain, aRect, ETrue );
    
    CreateWindowL();
    }


// ---------------------------------------------------------------------------
// CMccExAppBaseControl::SetRectAndActivateL
// ---------------------------------------------------------------------------
//
void CMccExAppBaseControl::SetRectAndActivateL( const TRect& aRect )
    {
    SetRect( aRect );
    ActivateL();
    }


// ---------------------------------------------------------------------------
// CMccExAppBaseControl::CMccExAppBaseControl
// ---------------------------------------------------------------------------
//
CMccExAppBaseControl::CMccExAppBaseControl()
    {
    }


// ---------------------------------------------------------------------------
// CMccExAppBaseControl::~CMccExAppBaseControl
// ---------------------------------------------------------------------------
//
CMccExAppBaseControl::~CMccExAppBaseControl()
    {
    delete iBgContext;
    }


// ---------------------------------------------------------------------------
// CMccExAppBaseControl::HandleResourceChange
// ---------------------------------------------------------------------------
//
void CMccExAppBaseControl::HandleResourceChange( TInt aType )
    {
    CCoeControl::HandleResourceChange( aType );
    
    // Handle spesifically the layout switch in here.
    TRect rect;
    if ( KEikDynamicLayoutVariantSwitch == aType &&
         AknLayoutUtils::LayoutMetricsRect(
         AknLayoutUtils::EMainPane, rect ) )
        {
        SetRect( rect );
        iBgContext->SetRect( rect );
        DrawDeferred();
        }
    }


// ---------------------------------------------------------------------------
// CMccExAppBaseControl::Draw
// ---------------------------------------------------------------------------
//
void CMccExAppBaseControl::Draw( const TRect& aRect ) const
    {
    MAknsSkinInstance* skin = AknsUtils::SkinInstance();
    MAknsControlContext* cc = AknsDrawUtils::ControlContext( this );
    AknsDrawUtils::Background( skin, cc, this, SystemGc(), aRect );
    }


// ---------------------------------------------------------------------------
// CMccExAppBaseControl::SizeChanged
// ---------------------------------------------------------------------------
//
void CMccExAppBaseControl::SizeChanged()
    {
    DrawDeferred();
    }


// -----------------------------------------------------------------------------
// CMccExAppBaseControl::MopSupplyObject
// -----------------------------------------------------------------------------
//  
TTypeUid::Ptr CMccExAppBaseControl::MopSupplyObject( TTypeUid aId )
    {
    if ( MAknsControlContext::ETypeId == aId.iUid ) 
        {
        return MAknsControlContext::SupplyMopObject( aId, iBgContext );
        }
    else
        {
        return CCoeControl::MopSupplyObject( aId );
        }
    }

