/*
* ============================================================================
*  Name        : mccexappui.cpp
*  Part of     : mccexample / mccexapp
*  Description : MCC example application UI class implementation
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.1.1
*/

#include "mccexapp.hrh"
#include "mccexpanic.pan"
#include "mccexappui.h"
#include "mccexappviewbase.h"
#include "mccexappmainview.h"
#include "mccexappaudioctrlview.h"
#include "mccexappvideoctrlview.h"
#include "mccexappnwsettingsview.h"
#include "mccexappcodecsettingview.h"
#include "mccexengine.h"

// ======== MEMBER FUNCTIONS ========

// ---------------------------------------------------------------------------
// CMccExAppUi::ConstructL
// ---------------------------------------------------------------------------
//
void CMccExAppUi::ConstructL()
    {
    BaseConstructL( EAknEnableSkin | EAknEnableMSK );
    iAppEngine = CMccExEngine::NewL();
    
    CMccExAppViewBase* view( NULL );
    view = CMccExAppMainView::NewLC( *iAppEngine );
    AddViewL( view );
    SetDefaultViewL( *view );
    CleanupStack::Pop( view );
    view = NULL;
    
    view = CMccExAppAudioCtrlView::NewLC( *iAppEngine );
    AddViewL( view );
    CleanupStack::Pop( view );
    view = NULL;
    
    view = CMccExAppVideoCtrlView::NewLC( *iAppEngine );
    AddViewL( view );
    CleanupStack::Pop( view );
    view = NULL;
    
    view = CMccExAppNwSettingsView::NewLC( *iAppEngine );
    AddViewL( view );
    CleanupStack::Pop( view );
    view = NULL;
    
    view = CMccExAppCodecSettingView::NewLC( *iAppEngine );
    AddViewL( view );
    CleanupStack::Pop( view );
    view = NULL;
    }


// ---------------------------------------------------------------------------
// CMccExAppUi::CMccExAppUi
// ---------------------------------------------------------------------------
//
CMccExAppUi::CMccExAppUi()
    {
    }


// ---------------------------------------------------------------------------
// CMccExAppUi::~CMccExAppUi
// ---------------------------------------------------------------------------
//
CMccExAppUi::~CMccExAppUi()
    {
    delete iAppEngine;
    }


// ---------------------------------------------------------------------------
// CMccExAppUi::HandleResourceChangeL
// ---------------------------------------------------------------------------
//
void CMccExAppUi::HandleResourceChangeL( TInt aType )
    {
    // NB: Always call CAknAppUiBase first whatever the resource change.
    CAknAppUiBase::HandleResourceChangeL( aType );
    if ( iView )
        {
        switch( iView->Id().iUid )
            {
            case EMccExAppViewDefault:
            case EMccExAppViewAudioCtrl:
            case EMccExAppViewVideoCtrl:
            case EMccExAppViewNwSettings:
            case EMccExAppViewCodecSettings:
                static_cast<CMccExAppViewBase*>( 
                        iView )->HandleResourceChange( aType );
                break;
            default:
                break;
            }
        }
    }


// ---------------------------------------------------------------------------
// CMccExAppUi::HandleCommandL
// ---------------------------------------------------------------------------
//
void CMccExAppUi::HandleCommandL( TInt aCommand )
    {
    switch ( aCommand )
        {
        case EAknSoftkeyBack:
            ActivateLocalViewL( TUid::Uid( EMccExAppViewDefault ) );
            break;
        case EEikCmdExit:
        case EAknSoftkeyExit:
            Exit();
            break;
        default:
            MccExPanic( EMccExUnhandledCmd );
            break;
        }
    }
