/*
* ============================================================================
*  Name        : mccexappvideocontainer.cpp
*  Part of     : mccexample / mccexapp
*  Description : MCC example CMccExAppVideoContainer implementation
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.1.1
*/

#include <aknutils.h>
#include <aknsdrawutils.h>

#include "mccexappvideocontainer.h"
#include "mvideocontrol.h"
#include "tvideodeviceparameters.h"
#include "mccexpanic.pan"

// Camera index 1 == front camera, 0 == main camera.
const TInt KMainCameraIndex( 1 );

// ======== MEMBER FUNCTIONS ========

// ---------------------------------------------------------------------------
// CMccExAppVideoContainer::NewL
// ---------------------------------------------------------------------------
//
CMccExAppVideoContainer* CMccExAppVideoContainer::NewL( const TRect& aRect )
    {
    CMccExAppVideoContainer* self = CMccExAppVideoContainer::NewLC( aRect );
    CleanupStack::Pop( self );
    return self;
    }


// ---------------------------------------------------------------------------
// CMccExAppVideoContainer::NewLC
// ---------------------------------------------------------------------------
//
CMccExAppVideoContainer* CMccExAppVideoContainer::NewLC( const TRect& aRect )
    {
    CMccExAppVideoContainer* self = new ( ELeave ) CMccExAppVideoContainer();
    CleanupStack::PushL( self );
    self->BaseConstructL( aRect );
    return self;
    }


// ---------------------------------------------------------------------------
// CMccExAppVideoContainer::CMccExAppVideoContainer
// ---------------------------------------------------------------------------
//
CMccExAppVideoContainer::CMccExAppVideoContainer()
    {
    }


// ---------------------------------------------------------------------------
// CMccExAppVideoContainer::~CMccExAppVideoContainer
// ---------------------------------------------------------------------------
//
CMccExAppVideoContainer::~CMccExAppVideoContainer()
    {
    }


// ---------------------------------------------------------------------------
// CMccExAppVideoContainer::SetVideoControl
// ---------------------------------------------------------------------------
//
void CMccExAppVideoContainer::SetVideoControl( MVideoControl* aControl )
    {
    // Assert the given parameter for not being NULL
    __ASSERT_ALWAYS( aControl, MccExPanic( EMccExNullPtr ) );
    iVideoControl = aControl;
    }


// ---------------------------------------------------------------------------
// CMccExAppVideoContainer::CreateVideoParametersL
// ---------------------------------------------------------------------------
//
void CMccExAppVideoContainer::CreateVideoParametersL(
    TVideoDeviceParameters& aParams )
    {
    CWsScreenDevice* scDev( NULL );
    scDev = static_cast<CWsScreenDevice*>( SystemGc().Device() );
    if ( scDev )
        {
        aParams.iWindowOrdinalPosition = DrawableWindow()->OrdinalPosition();
        aParams.iWindowOrdinalPriority = ECoeWinPriorityLow;
        aParams.iCameraIndex = KMainCameraIndex;
        aParams.iScreenIndex = scDev->GetScreenNumber();
        }
    else
        {
        User::Leave( KErrNotReady );
        }
    }


// ---------------------------------------------------------------------------
// CMccExAppVideoContainer::Draw
// ---------------------------------------------------------------------------
//
void CMccExAppVideoContainer::Draw( const TRect& aRect ) const
    {
    // Update ("draw") the video screen. Get the screen rotation of the
    // screen from screen device because CAknAppUiBase::Orientation does not
    // seem to work properly.
    TInt rotation( KScreenRotation0 );
    CWsScreenDevice* scDev( NULL );
    scDev = static_cast<CWsScreenDevice*>( SystemGc().Device() );
    if ( scDev )
        {
        const TInt mode( scDev->CurrentScreenMode() );
        TPixelsAndRotation pixAndRot;
        scDev->GetScreenModeSizeAndRotation( mode, pixAndRot );
        
        if ( CFbsBitGc::EGraphicsOrientationRotated90 == pixAndRot.iRotation )
            {
            rotation = KScreenRotation90;
            }
        else if ( CFbsBitGc::EGraphicsOrientationRotated180 ==
                  pixAndRot.iRotation )
            {
            rotation = KScreenRotation180;
            }
        else if ( CFbsBitGc::EGraphicsOrientationRotated270 ==
                  pixAndRot.iRotation )
            {
            rotation = KScreenRotation270;
            }
        }
    
    if ( iVideoControl )
        {
        TInt ordinalPos( Window().OrdinalPosition() );
        
        // If the control's window position is 1, set the video screen to use
        // position 0 (topmost).
        if ( 1 == ordinalPos )
            {
            ordinalPos = 0;
            }
        else
            {
            ordinalPos += 1;
            }
        
        // Try to obtain the main pane rect, if it is not available use the
        // given rect.
        TRect rect;
        if ( !AknLayoutUtils::LayoutMetricsRect(
             AknLayoutUtils::EMainPane, rect ) )
            {
            rect = aRect;
            }
        
        TRAP_IGNORE( iVideoControl->UpdateScreenRectL( rect, ordinalPos,
                    Window().OrdinalPriority(), rotation ) )
        }
    
    // Then - draw the background
    CMccExAppBaseControl::Draw( aRect );
    }


// ---------------------------------------------------------------------------
// CMccExAppVideoContainer::HandleResourceChange
// ---------------------------------------------------------------------------
//
void CMccExAppVideoContainer::HandleResourceChange( TInt aType )
    {
    CCoeControl::HandleResourceChange( aType );
    
    // Handle spesifically the layout switch in here.
    TRect rect;
    if ( AknLayoutUtils::LayoutMetricsRect( AknLayoutUtils::EMainPane, rect ) )
        {
        SetRect( rect );
        DrawNow();
        }
    }

