/*
* ============================================================================
*  Name        : downlinkaudiostream.h
*  Part of     : mccexample / mccexengine
*  Description : CDownlinkAudioStream declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CDOWNLINKAUDIOSTREAM_H
#define C_CDOWNLINKAUDIOSTREAM_H

#include <e32base.h>

#include "audiostreambase.h"

class CMccInterface;
class CMccCodecInformation;
class CStreamStateMachine;

/**
 * CDownlinkAudioStream provides a implementation of downlink direction audio
 * stream. In short this class can be used to create and control an audio
 * stream from IP network to device audio output (earpiece or loudspeaker).
 */
NONSHARABLE_CLASS( CDownlinkAudioStream ) : public CAudioStreamBase
    {
public:
    
    /**
     * Destructor.
     */
    virtual ~CDownlinkAudioStream();

    /**
     * Two-phased constructor.
     * 
     * @param aStateMachine Reference to the state machine.
     * @param aCodec Reference to a codec which should be used for audio.
     * @param aMcc Reference to the MCC.
     * @param aSession ID of the session.
     * @param aLink ID of the link.
     */
    static CDownlinkAudioStream* NewL( CStreamStateMachine& aStateMachine,
            CMccCodecInformation& aCodec, CMccInterface& aMcc,
            TUint32 aSession, TUint32 aLink );

    /**
     * Two-phased constructor. Leaves the created instance into the cleanup
     * stack.
     * 
     * @param aStateMachine Reference to the state machine.
     * @param aCodec Reference to a codec which should be used for audio.
     * @param aMcc Reference to the MCC.
     * @param aSession ID of the session.
     * @param aLink ID of the link.
     */
    static CDownlinkAudioStream* NewLC( CStreamStateMachine& aStateMachine,
            CMccCodecInformation& aCodec, CMccInterface& aMcc,
            TUint32 aSession, TUint32 aLink );
    
    /**
     * Returns the current audio output routing of this stream.
     * 
     * @leave Standard Symbian OS error code.
     * @see mmccinterfacedef.h
     * @return TUint32 Audio routing.
     */
    TUint32 CurrentAudioRouteL();
    
    /**
     * Sets the current audio output routing of this stream.
     * 
     * @leave Standard Symbian OS error code.
     * @see mmccinterfacedef.h
     * @param aRoute New audio route.
     */
    void SetCurrentAudioRouteL( TUint32 aRoute );
    
    /**
     * Returns the current audio volume of this stream.
     * 
     * @leave Standard Symbian OS error code.
     * @see mmccinterfacedef.h
     * @return TInt Current volume level.
     */
    TInt VolumeL();
    
    /**
     * Sets the current audio volume of this stream.
     * 
     * @leave Standard Symbian OS error code.
     * @see mmccinterfacedef.h
     * @param aVolume New volume level.
     */
    void SetVolumeL( TInt aVolume );
    
    /**
     * Returns the maximum audio volume of this stream.
     * 
     * @leave Standard Symbian OS error code.
     * @see mmccinterfacedef.h
     * @return TInt Maximum volume level.
     */
    TInt MaxVolumeL();
    
protected:
    
// From CStreamBase
    
    /**
     * From CStreamBase.
     * Finalizes the stream creation.
     * 
     * @see CStreamBase
     */
    void CreateStreamL();
    
private:
    
    /**
     * Constructor for performing 1st stage construction.
     * 
     * @param aStateMachine Reference to the state machine.
     * @param aMcc Reference to the MCC.
     * @param aSession ID of the session.
     * @param aLink ID of the link.
     */
    CDownlinkAudioStream( CStreamStateMachine& aStateMachine,
            CMccInterface& aMcc, TUint32 aSession, TUint32 aLink );
    
    };

#endif  // C_CDOWNLINKAUDIOSTREAM_H
