/*
* ============================================================================
*  Name        : streamstatemachine.h
*  Part of     : mccexample / mccexengine
*  Description : CStreamStateMachine declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CSTREAMSTATEMACHINE_H
#define C_CSTREAMSTATEMACHINE_H

#include <e32base.h>
#include "tcontrolstate.h"

class CStreamStateBase;
typedef RPointerArray<CStreamStateBase> RStateArray;


/**
 * CStreamStateMachine provides a basic state machine for MCC stream state
 * changes.
 */
NONSHARABLE_CLASS( CStreamStateMachine ) : public CBase
    {
public:
    
    /**
     * Destructor.
     */
    virtual ~CStreamStateMachine();
    
    /**
     * Two-phased constructor.
     */
    static CStreamStateMachine* NewL();
    
    /**
     * Two-phased constructor. Leaves the created instance into the cleanup
     * stack.
     */
    static CStreamStateMachine* NewLC();
    
    /**
     * Returns a state object associated for the given control state.
     * Ownership is not transferred.
     * 
     * @leave KErrNotFound If the requested state is not found.
     */
    CStreamStateBase& RequestStateL( TControlState aRequested );
    
private:
    
    /**
     * Constructor for performing 1st stage construction.
     */
    CStreamStateMachine();
    
    /**
     * Symbian default constructor for performing 2nd stage construction.
     */
    void ConstructL();
    
private:    // data
    
    /**
     * Array of possible states.
     * Own.
     */
    RStateArray iStates;
    
    };

#endif  // C_CSTREAMSTATEMACHINE_H
