/*
* ============================================================================
*  Name        : mcceventhandler.cpp
*  Part of     : mccexample / mccexengine
*  Description : MCC example event handler implementation
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.1.1
*/

#include "mcceventhandler.h"
#include "mccnetworksession.h"

// ======== MEMBER FUNCTIONS ========

// ---------------------------------------------------------------------------
// CMccEventHandler::CMccEventHandler
// ---------------------------------------------------------------------------
//
CMccEventHandler::CMccEventHandler()
    {
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::~CMccEventHandler
// ---------------------------------------------------------------------------
//
CMccEventHandler::~CMccEventHandler()
    {
    // Sessions are not owned, so we do NOT call ResetAndDestroy().
    iSessionArray.Close();
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::NewLC
// ---------------------------------------------------------------------------
//
CMccEventHandler* CMccEventHandler::NewLC()
    {
    CMccEventHandler* self = new (ELeave) CMccEventHandler();
    CleanupStack::PushL( self );
    return self;
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::NewL
// ---------------------------------------------------------------------------
//
CMccEventHandler* CMccEventHandler::NewL()
    {
    CMccEventHandler* self = CMccEventHandler::NewLC();
    CleanupStack::Pop( self );
    return self;
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::AddSessionL
// ---------------------------------------------------------------------------
//
void CMccEventHandler::AddSessionL( CMccNetworkSession* aSession )
    {
    // Assert the proper argument and that we do not already have it stored.
    __ASSERT_ALWAYS( aSession, User::Leave( KErrArgument ) );
    __ASSERT_ALWAYS( KErrNotFound == iSessionArray.Find( aSession ),
            User::Leave( KErrAlreadyExists ) );
    
    iSessionArray.AppendL( aSession );
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::RemoveSession
// ---------------------------------------------------------------------------
//
void CMccEventHandler::RemoveSession( CMccNetworkSession* aSession )
    {
    if ( aSession )
        {
        const TInt index( iSessionArray.Find( aSession ) );
        if ( KErrNotFound != index )
            {
            iSessionArray.Remove( index );
            }
        }
    }

// ---------------------------------------------------------------------------
// CMccEventHandler::MccEventReceived
// ---------------------------------------------------------------------------
//
void CMccEventHandler::MccEventReceived( const TMccEvent& aEvent )
    {
    // Filter out playing and buffering events because they cause _very_ much
    // noise especially in video streaming.
    if ( KMccStreamPlaying != aEvent.iEventType && 
         KMccStreamBuffering != aEvent.iEventType &&
         KErrNone == aEvent.iErrorCode )
        {
        CMccNetworkSession* session = 
            FindMatchingSession( aEvent.iSessionId, aEvent.iLinkId );
        
        if ( session && KMccStreamPlaying != aEvent.iEventType && 
             KMccStreamBuffering != aEvent.iEventType )
            {
            session->HandleEvent( aEvent );
            }
        }
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::MccMediaStarted
// ---------------------------------------------------------------------------
//
void CMccEventHandler::MccMediaStarted( TUint32 aSessionId,
    TUint32 aLinkId,     
    TUint32 aStreamId,
    TUint32 aSinkSourceId )
    {
    CMccNetworkSession* session = FindMatchingSession( aSessionId, aLinkId );
    if ( session )
        {
        TMccEvent event;
        event.iSessionId = aSessionId;
        event.iLinkId = aLinkId;
        event.iStreamId = aStreamId;
        event.iEndpointId = aSinkSourceId;
        event.iEventCategory = KMccEventCategoryStream;
        event.iErrorCode = KErrNone;
        event.iEventType = KMccStreamStarted;
        
        session->HandleEvent( event );
        }
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::MccMediaStopped
// ---------------------------------------------------------------------------
//
void CMccEventHandler::MccMediaStopped( TUint32 aSessionId,
    TUint32 aLinkId,     
    TUint32 aStreamId,
    TUint32 aSinkSourceId )
    {
    CMccNetworkSession* session = FindMatchingSession( aSessionId, aLinkId );
    if ( session )
        {
        TMccEvent event;
        event.iSessionId = aSessionId;
        event.iLinkId = aLinkId;
        event.iStreamId = aStreamId;
        event.iEndpointId = aSinkSourceId;
        event.iEventCategory = KMccEventCategoryStream;
        event.iErrorCode = KErrNone;
        event.iEventType = KMccStreamStopped;
        
        session->HandleEvent( event );
        }
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::MccMediaPaused
// ---------------------------------------------------------------------------
//
void CMccEventHandler::MccMediaPaused( TUint32 aSessionId,
    TUint32 aLinkId,     
    TUint32 aStreamId,
    TUint32 aSinkSourceId )
    {
    CMccNetworkSession* session = FindMatchingSession( aSessionId, aLinkId );
    if ( session )
        {
        TMccEvent event;
        event.iSessionId = aSessionId;
        event.iLinkId = aLinkId;
        event.iStreamId = aStreamId;
        event.iEndpointId = aSinkSourceId;
        event.iEventCategory = KMccEventCategoryStream;
        event.iErrorCode = KErrNone;
        event.iEventType = KMccStreamPaused;
        
        session->HandleEvent( event );
        }
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::MccMediaResumed
// ---------------------------------------------------------------------------
//
void CMccEventHandler::MccMediaResumed( TUint32 aSessionId,
    TUint32 aLinkId,     
    TUint32 aStreamId,
    TUint32 aSinkSourceId )
    {
    CMccNetworkSession* session = FindMatchingSession( aSessionId, aLinkId );
    if ( session )
        {
        TMccEvent event;
        event.iSessionId = aSessionId;
        event.iLinkId = aLinkId;
        event.iStreamId = aStreamId;
        event.iEndpointId = aSinkSourceId;
        event.iEventCategory = KMccEventCategoryStream;
        event.iErrorCode = KErrNone;
        event.iEventType = KMccStreamResumed;
        
        session->HandleEvent( event );
        }
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::MccMediaPrepared
// ---------------------------------------------------------------------------
//
void CMccEventHandler::MccMediaPrepared( TUint32 aSessionId,
    TUint32 aLinkId,     
    TUint32 aStreamId,
    TUint32 aSinkSourceId )
    {
    CMccNetworkSession* session = FindMatchingSession( aSessionId, aLinkId );
    if ( session )
        {
        TMccEvent event;
        event.iSessionId = aSessionId;
        event.iLinkId = aLinkId;
        event.iStreamId = aStreamId;
        event.iEndpointId = aSinkSourceId;
        event.iEventCategory = KMccEventCategoryStream;
        event.iErrorCode = KErrNone;
        event.iEventType = KMccStreamPrepared;
        
        session->HandleEvent( event );
        }
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::MccMediaInactive
// ---------------------------------------------------------------------------
//
void CMccEventHandler::MccMediaInactive( TUint32 aSessionId,
    TUint32 aLinkId,     
    TUint32 aStreamId,
    TUint32 aSinkSourceId )
    {
    CMccNetworkSession* session = FindMatchingSession( aSessionId, aLinkId );
    if ( session )
        {
        TMccEvent event;
        event.iSessionId = aSessionId;
        event.iLinkId = aLinkId;
        event.iStreamId = aStreamId;
        event.iEndpointId = aSinkSourceId;
        event.iEventCategory = KMccEventCategoryStream;
        event.iErrorCode = KErrNone;
        event.iEventType = KMccInactivityEvent;
        
        session->HandleEvent( event );
        }
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::MccMediaActive
// ---------------------------------------------------------------------------
//
void CMccEventHandler::MccMediaActive( TUint32 aSessionId,
    TUint32 aLinkId,     
    TUint32 aStreamId,
    TUint32 aSinkSourceId )
    {
    CMccNetworkSession* session = FindMatchingSession( aSessionId, aLinkId );
    if ( session )
        {
        TMccEvent event;
        event.iSessionId = aSessionId;
        event.iLinkId = aLinkId;
        event.iStreamId = aStreamId;
        event.iEndpointId = aSinkSourceId;
        event.iEventCategory = KMccEventCategoryStream;
        event.iErrorCode = KErrNone;
        event.iEventType = KMccActivityEvent;
        
        session->HandleEvent( event );
        }
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::MccCtrlError
// ---------------------------------------------------------------------------
//
void CMccEventHandler::MccCtrlError( TInt aError,
    TUint32 aSessionId,
    TUint32 aLinkId,     
    TUint32 aStreamId,
    TUint32 aSinkSourceId )
    {
    CMccNetworkSession* session = FindMatchingSession( aSessionId, aLinkId );
    if ( session )
        {
        TMccEvent event;
        event.iSessionId = aSessionId;
        event.iLinkId = aLinkId;
        event.iStreamId = aStreamId;
        event.iEndpointId = aSinkSourceId;
        event.iEventCategory = KMccEventCategoryStream;
        event.iErrorCode = aError;
        event.iEventType = KMccStreamError;
        
        session->HandleEvent( event );
        }
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::UnknownMediaReceived
// ---------------------------------------------------------------------------
//
void CMccEventHandler::UnknownMediaReceived( TUint32 aSessionId,
    TUint32 aLinkId,     
    TUint32 aStreamId,
    TUint32 aSinkSourceId,
    TUint8 aPayloadType )
    {
    CMccNetworkSession* session = FindMatchingSession( aSessionId, aLinkId );
    if ( session )
        {
        TMccEvent event;
        event.iSessionId = aSessionId;
        event.iLinkId = aLinkId;
        event.iStreamId = aStreamId;
        event.iEndpointId = aSinkSourceId;
        event.iEventCategory = KMccEventCategoryStream;
        event.iErrorCode = KErrNone;
        event.iEventType = KMccUnknownMediaReceived;
        event.iEventNumData = aPayloadType;
        
        session->HandleEvent( event );
        }
    }


// ---------------------------------------------------------------------------
// CMccEventHandler::FindMatchingSession
// ---------------------------------------------------------------------------
//
CMccNetworkSession* CMccEventHandler::FindMatchingSession( TUint32 aSessionId,
    TUint32 aLinkId ) const
    {
    TInt count( iSessionArray.Count() );
    while( count )
        {
        count -= 1;
        if ( iSessionArray[count]->Matches( aSessionId, aLinkId ) )
            {
            return iSessionArray[count];
            }
        }
    
    return NULL;
    }
